#pragma once

// Folgender ifdef-Block ist die Standardmethode zum Erstellen von Makros, die das Exportieren 
// aus einer DLL vereinfachen. Alle Dateien in dieser DLL werden mit dem BLUEDATWIN32INTERFACE_EXPORTS-Symbol
// (in der Befehlszeile definiert) kompiliert. Dieses Symbol darf fr kein Projekt definiert werden,
// das diese DLL verwendet. Alle anderen Projekte, deren Quelldateien diese Datei beinhalten, erkennen 
// BLUEDATWIN32INTERFACE_API-Funktionen als aus einer DLL importiert, whrend die DLL
// mit diesem Makro definierte Symbole als exportiert ansieht.
#ifdef BLUEDATWIN32INTERFACE_EXPORTS
#define BLUEDATWIN32INTERFACE_API extern "C" __declspec(dllexport)
#else
#define BLUEDATWIN32INTERFACE_API extern "C" __declspec(dllimport)
#endif

#define MIN_BUFFER_SIZE_SERIAL	16		// Mindestgre fr den Textpuffer zum Abfragen der Seriennummer
#define MIN_BUFFER_SIZE_TYPE	32		// Mindestgre fr den Textpuffer zum Abfragen des Sendertyps
#define MIN_BUFFER_SIZE_STATUS	256		// Mindestgre fr den Textpuffer zum Abfragen von Statusinformationen

#define LED_NONE		0				// Alle LEDs aus	
#define LED_GREEN		1				// Grne LED
#define LED_RED			2				// Rote LED
#define LED_RED_GREEN	3				// Grne und Rote LED

#define STATE_CONNECTED		0			// Status Verbunden
#define STATE_DISCONNECTED	1			// Status Getrennt

#define SWITCH_OFF_TIME_MIN 2			// Mindestens 2 Sekunden
#define SWITCH_OFF_TIME_MAX 1800		// Maximal 30 Minuten

// Fehlercodes
enum BLUEDAT_ERROR {
	BD_NO_ERROR = 0,						// Kein Fehler
	BD_UNKNOWN_ERROR = -1,					// Unbekannter Fehler
	BD_ERROR_INITIALIZE = -2,				// Die Initialisierung ist fehlgeschlagen
	BD_ERROR_INVALID_PARAMETER = -3,		// Ungltiger Parameter
	BD_ERROR_INVALID_RETURN_VALUE = -4,		// Ungltiger Rckgabewert
	BD_ERROR_BUFFER_TOO_SMALL = -5,			// Textpuffer ist zu klein
	BD_ERROR_CONVERT_STRING = -6,			// Unicode Text konnte nicht in Ansi Text konvertiert werden
	BD_ERROR_INITIALIZE_COM_OBJECT = -7,	// Das COM-Interface kann nicht initialisiert werden
	BD_ERROR_ADD_TRANSMITTER = -8,			// Der Transmitter kann nicht hinzugefgt werden
	BD_ERROR_REMOVE_TRANSMITTER = -9,		// Der Transmitter kann nicht entfernt werden
	BD_ERROR_SHUT_DOWN_TRANSMITTER = -10,	// Der Transmitter kann nicht ausgeschaltet werden
	BD_ERROR_SEND_ACKNOWLEDGE = -11,		// Die Rckmeldung kann nicht gesendet werden
	BD_ERROR_REQUEST_VALUE = -12,			// Es konnte kein Messwert angefordert werden
	BD_ERROR_SET_SWITCH_OFF_TIME = -13,		// Die Ausschaltzeit kann nicht gesetzt werden
	BD_ERROR_GET_SWITCH_OFF_TIME = -14,		// Die Ausschaltzeit kann nicht ermittelt werden
	BD_ERRPR_RECEIVER_NOT_CONNECTED = -15,	// Kein Empfnger angeschlossen
	BD_ERROR_TRANSMITTER_OFFLINE = -16,		// Sender ausgeschaltet
	BD_ERROR_TRANSMITTER_NOT_IN_LIST = -17,	// Sender nicht in Gerteliste
};

/// <summary>
/// Definition der Typen fr die CallBack-Funktionen
/// </summary>
typedef int (__stdcall *NEWMEASUREMENTVALUE) (const char* szSerial, const char* szValue);

typedef int(__stdcall *TRANSMITTERSTATECHANGED) (const char* szSerial, int iNewState);

typedef int(__stdcall *RECEIVERSTATECHANGED) (int iNewState);

/// <summary>
/// bergibt der DLL die Adressen der CallBack-Funktionen, die bei verschiedenen Events aufgerufen werden sollen
/// </summary>
/// <param name="pNewMeasurementValueFct">Adresse der Funktion, die aufgerufen wird, wenn ein neuer Messwert empfangen wird</param>
/// <param name="pTransmitterStateChangedFct">Adresse der Funktion, die aufgerufen wird, wenn sich der Status eines Senders verndert</param>
/// <param name="pReceiverStateChangedFct">Adresse der Funktion, die aufgerufen wird, wenn sich der Status des Empfngers verndert</param>
/// <returns> 0 = Kein Fehler; <0 = Fehlercode</returns>
BLUEDATWIN32INTERFACE_API int RegisterCallBackFunctions(NEWMEASUREMENTVALUE pNewMeasurementValueFct=NULL, TRANSMITTERSTATECHANGED pTransmitterStateChangedFct=NULL, RECEIVERSTATECHANGED pReceiverStateChangedFct=NULL);

/// <summary>
/// Fragt den Status ab, ob neue Gerte gefunden werden sollen oder nicht
/// </summary>
/// <returns> >0 = Es sollen neue Gerte gefunden werden; 0 = Es sollen keine neuen Gerte gefunden werden; <0 = Fehlercode</returns>
BLUEDATWIN32INTERFACE_API int GetSearchMode(void);

/// <summary>
/// Setzt den Status, ob neue Gerte gefunden werden sollen oder nicht
/// </summary>
/// <param name="bValue">true: Es sollen neue Gerte gefunden werden; false: Es sollen keine neuen Gerte gefunden werden</param>
/// <returns>0 = kein Fehler, <0 = Fehlercode</returns>
BLUEDATWIN32INTERFACE_API int SetSearchMode(bool bValue);

/// <summary>
/// Liefert die Anzahl der gefundenen Sender.
/// </summary>
/// <returns> >0 = Anzahl der gefundenen Sender; 0 = kein Sender; <0 = Fehlercode</returns>
BLUEDATWIN32INTERFACE_API int GetNewTransmitterCount(void);

/// <summary>
/// Liefert zu einem Index die entsprechende Seriennummer.
/// </summary>
/// <param name="iIndex">Index des gefundenen Senders</param>
/// <param name="pBuffer">Textpuffer zur Aufnahme der Seriennummer</param>
/// <param name="iLength">Lnge des Textpuffers</param>
/// <returns>0 = kein Fehler, <0 = Fehlercode</returns>
BLUEDATWIN32INTERFACE_API int GetNewTransmitterSerialnumber(int iIndex, char* pBuffer, int iLength);

/// <summary>
/// Fgt einen Sender hinzu mit dem sich der Empfnger verbinden soll.
/// </summary>
/// <param name="szSerial">Seriennummer des Senders</param>
/// <returns>0 = kein Fehler, <0 = Fehlercode</returns>
BLUEDATWIN32INTERFACE_API int AddTransmitter(const char* szSerial);

/// <summary>
/// Entfernt einen Sender. Dieser wird dann nicht mehr verbunden.
/// </summary>
/// <param name="szSerial">Seriennummer des Senders</param>
/// <returns>0 = kein Fehler, <0 = Fehlercode</returns>
BLUEDATWIN32INTERFACE_API int RemoveTransmitter(const char* szSerial);

/// <summary>
/// Sendet eine Rckmeldung an den Sender
/// </summary>
/// <param name="szSerial">Seriennummer des Senders</param>
/// <param name="iLed">LED die am Sender aufleuchten soll</param>
/// <returns>0 = kein Fehler, <0 = Fehlercode</returns>
BLUEDATWIN32INTERFACE_API int SendAcknowledge(const char* szSerial, int iLed = LED_GREEN);

/// <summary>
/// Fragt einen Messwert am Sender ab.
/// </summary>
/// <param name="szSerial">Seriennummer des Senders</param>
/// <param name="iLed">LED die am Sender bei der Anfrage aufleuchten soll</param>
//TODO: Parameter iLed definieren!!!
/// <returns>0 = kein Fehler, <0 = Fehlercode</returns>
BLUEDATWIN32INTERFACE_API int RequestMeasurementValue(const char* szSerial, int iLed = LED_NONE);

/// <summary>
/// Schaltet den Sender aus.
/// </summary>
/// <param name="szSerial">Seriennummer des Senders</param>
/// <returns>0 = kein Fehler, <0 = Fehlercode</returns>
BLUEDATWIN32INTERFACE_API int ShutDownTransmitter(const char* szSerial);

/// <summary>
/// Setzt die Zeit bis der Sender sich ausschaltet.
/// </summary>
/// <param name="szSerial">Seriennummer des Senders</param>
/// <param name="iTime">Zeit in Sekunden</param>
/// <returns>0 = kein Fehler, <0 = Fehlercode</returns>
BLUEDATWIN32INTERFACE_API int SetSwitchOffTime(const char* szSerial, int iTime = 20);

/// <summary>
/// Liefert die Zeit bis der Sender sich ausschaltet (in Sekunden).
/// </summary>
/// <param name="szSerial">Seriennummer des Senders</param>
/// <returns> >0 = Zeit in Sekunden; 0 = keine Zeit; <0 = Fehlercode</returns>
BLUEDATWIN32INTERFACE_API int GetSwitchOffTime(const char* szSerial);

/// <summary>
/// Liefert den Typ eines Senders (z.b. RS232, 16EX).
/// </summary>
/// <param name="szSerial">Seriennummer des Senders</param>
/// <param name="pBuffer">Textpuffer zur Aufnahme des Typs</param>
/// <param name="iLength">Lnge des Textpuffers</param>
/// <returns>0 = kein Fehler, <0 = Fehlercode</returns>
BLUEDATWIN32INTERFACE_API int GetTransmitterType(const char* szSerial, char* pBuffer, int iLength);

/// <summary>
/// Unbenutzt
/// </summary>
/// <param name="szSerial">Seriennummer des Senders</param>
/// <param name="szParam">Funktion oder Einstellung</param>
/// <param name="pBuffer">Textpuffer zur Aufnahme des Typs</param>
/// <param name="iLength">Lnge des Textpuffers</param>
/// <returns>0 = kein Fehler, <0 = Fehlercode</returns>
BLUEDATWIN32INTERFACE_API int GetState(const char* szSerial, const char* szParam, char* pBuffer, int iLength);
